//--------------------------------------------------------------
// File     : main.c
// Datum    : 17.10.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : CMSIS_BOOT, M4_CMSIS_CORE
// Funktion : Capture-Client
//            > ein PC muss per USB-OTG angeschlossen werden
//            > auf dem PC muss die PC-Software "Capture-Tool"
//              gestartet werden
//            > Die PC-Software sendet Bild-Daten an die CPU
//              die CPU zeigt diese auf dem LCD an
//            > mit dem User-Button kann der Capture-Mode
//              verlassen werden
//
// Hinweis  : Diese zwei Files muessen auf 8MHz stehen
//              "cmsis_boot/stm32f4xx.h"
//              "cmsis_boot/system_stm32f4xx.c"
//
// Librarys : Die eingesetzen Library fr USB-CDC ist leicht
//            abgendert (Funktion "VCP_DataRx" in "usbd_cdc_vcp.c")
//
//--------------------------------------------------------------

#include "main.h"
#include "stm32_ub_lcd_st7783.h"
#include "stm32_ub_usb_cdc.h"
#include "stm32_ub_button.h"
#include <string.h>

//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_Check_Kommand(char *ptr);
void P_setMode(char *ptr);
void P_setWindow(char *ptr, uint8_t mode);
uint32_t P_STR2HEX(char *ptr, uint8_t start, uint8_t len);
uint8_t P_ASC2INT(uint8_t ascii);


//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
uint32_t ascii_err_flag=0;
uint8_t capture_mode=0;  // 0=inaktiv, 1=aktiv
uint16_t win_mode,window_w, window_h;
uint8_t err_mode;
uint8_t link_status; // 0=getrennt, 1=gesteckt, 2=verbunden



//--------------------------------------------------------------
int main(void)
{
  USB_CDC_STATUS_t check_usb;
  char rx_string[30];

  SystemInit(); // Quarz Einstellungen aktivieren

  capture_mode=0;      // capture inaktiv
  link_status=0;       // 0=getrennt

  // Init vom Button
  UB_Button_Init();

  // Init vom LCD
  UB_LCD_Init();
  UB_LCD_FillScreen(RGB_COL_GREY);

  // init vom CDC
  UB_USB_CDC_Init();

  while(1)
  {
    // test ob USB mit PC verbunden ist
    check_usb=UB_USB_CDC_GetStatus();
    if(check_usb==USB_CDC_CONNECTED) {
      if(link_status==0) {
        link_status=1; // 1=gesteckt
        UB_LCD_FillScreen(RGB_COL_MAGENTA);
      }
      if(capture_mode==0) {
        //--------------------------
        // normaler Mode
        //--------------------------
        if(UB_USB_CDC_ReceiveString(rx_string)==RX_READY) {
          // empfangenes Kommando auswerten
          P_Check_Kommand(rx_string);
        }
      }
      else {
        //--------------------------
        // capture Mode
        //--------------------------    
        if(UB_Button_OnClick(BTN_USER)==true) {
          // Capture-Mode beenden
          capture_mode=0;
          UB_LCD_SetWindow(0,0,LCD_MAXX-1,LCD_MAXY-1);
          UB_LCD_FillScreen(RGB_COL_GREY);
          UB_USB_CDC_SendString("CT_STOP_OK",CR);
        }
      }
    }
    else {
      // keine USB-Verbindung
      if(link_status!=0) {
        link_status=0; // 0=getrennt
        // Capture-Mode beenden
        capture_mode=0;
        UB_LCD_SetWindow(0,0,LCD_MAXX-1,LCD_MAXY-1);
        UB_LCD_FillScreen(RGB_COL_GREY);
      }
    }
  }
}


//--------------------------------------------------------------
// empfangene Kommandos auswerten
//--------------------------------------------------------------
void P_Check_Kommand(char *ptr)
{
  if(strcmp(ptr,"CT_SEARCH")==0) {
    UB_LCD_FillScreen(RGB_COL_YELLOW);
    UB_USB_CDC_SendString("CT_SEARCH_OK",CR);
    link_status=2; // 2=verbunden
  }
  else if(strcmp(ptr,"CT_SETTINGS")==0) {
    err_mode=1;
    win_mode=0;
    window_w=0;
    window_h=0;
  }
  else if(strncmp(ptr,"CT_MODE=xxxx",8)==0) {
    P_setMode(ptr);
  }
  else if(strncmp(ptr,"CT_WIDTH=xxxx",9)==0) {
    P_setWindow(ptr,0);
  }
  else if(strncmp(ptr,"CT_HEIGHT=xxxx",10)==0) {
    P_setWindow(ptr,1);
  }
  else if(strcmp(ptr,"CT_AKTIVE")==0) {
    if(err_mode==0) {
      UB_LCD_FillScreen(RGB_COL_GREEN);
      UB_LCD_SetCursor2Draw(0,0);
      UB_USB_CDC_SendString("CT_AKTIVE_OK",CR);
      capture_mode=1;
    }
    else {
      UB_LCD_FillScreen(RGB_COL_RED);
      UB_USB_CDC_SendString("CT_SETTINGS_ERR",CR);
    }
  }
}


//--------------------------------------------------------------
// Display Mode einstellen
//--------------------------------------------------------------
void P_setMode(char *ptr)
{
  uint32_t wert;

  wert=P_STR2HEX(ptr,8,4);
  if(ascii_err_flag==0) {
    if(wert==0) {
      UB_LCD_SetMode(PORTRAIT);
      win_mode=0;
    }
    else {
      UB_LCD_SetMode(LANDSCAPE);
      win_mode=1;
    }
  }
}


//--------------------------------------------------------------
// Display Auflsung einstellen
//--------------------------------------------------------------
void P_setWindow(char *ptr, uint8_t mode)
{
  uint32_t wert;

  if(mode==0) {
    wert=P_STR2HEX(ptr,9,4);
    if(ascii_err_flag==0) {
      window_w=wert;
    }
  }
  else {
    wert=P_STR2HEX(ptr,10,4);
    if(ascii_err_flag==0) {
      window_h=wert;
    }

    err_mode=1;
    if(win_mode==0) {
      if((window_w!=0) && (window_h!=0)) {
        if((window_w<=LCD_MAXX) && (window_h<=LCD_MAXY)) {
          UB_LCD_SetWindow(0, 0, window_w-1, window_h-1);
          err_mode=0; // keine Fehler
        }
      }
    }
    else {
      if((window_w!=0) && (window_h!=0)) {
        if((window_w<=LCD_MAXY) && (window_h<=LCD_MAXX)) {
          UB_LCD_SetWindow(0, 0, window_h-1, window_w-1);
          err_mode=0; // keine Fehler
        }
      }
    }
  }
}


//--------------------------------------------------------------
// Teilstring in Zahl umwandeln
//  "HALLO_1A",6,2, -> 1Ah = 26d
//--------------------------------------------------------------
uint32_t P_STR2HEX(char *ptr, uint8_t start, uint8_t len)
{
  uint32_t ret_wert=0;
  uint8_t wert;
  char *p;

  ascii_err_flag=0;

  p=ptr+start;
  if(len==1) {
    wert=P_ASC2INT(*p);
    ret_wert=wert;
  }
  else if(len==2) {
    wert=P_ASC2INT(*p);
    ret_wert=wert*0x10;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert;
  }
  else if(len==3) {
    wert=P_ASC2INT(*p);
    ret_wert=wert*0x100;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert*0x10;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert;
  }
  else if(len==4) {
    wert=P_ASC2INT(*p);
    ret_wert=wert*0x1000;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert*0x100;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert*0x10;
    p++;
    wert=P_ASC2INT(*p);
    ret_wert+=wert;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// Ascii-Zeichen in Integer umwandeln
// 0 -> 0
// 9 -> 9
// a -> 10
// f -> 15
// A -> 10
// F -> 15
// sonst -> 0 + error_flag
//--------------------------------------------------------------
uint8_t P_ASC2INT(uint8_t ascii)
{
  uint8_t ret_wert=0;

  if((ascii>=48) && (ascii<=57)) {
    ret_wert=ascii-48;
  }
  else if((ascii>=65) && (ascii<=70)) {
    ret_wert=ascii-55;
  }
  else if((ascii>=97) && (ascii<=102)) {
    ret_wert=ascii-87;
  }
  else {
    ascii_err_flag=1;
  }

  return(ret_wert);
}


